'+-----------------------------------------------------------------------------+
'|                                                                             |
'| Autor: Jonas Eberhard   www.joseng.de                                       |
'| Datei: DMX LED Dimmer 1Kanal 1kHz.bas                                       |
'| Beschreibung: LED Dimmer 1 Kanal per DMX mit einer PWM Frequenz von 1kHz    |
'|                                                                             |
'| Prozessor: ATtiny2313                                                       |
'| Fuse Bits:  LowByte: 0xFF    HighByte: 0xDF    EByte: 0xFF                  |
'|                                                                             |
'+-----------------------------------------------------------------------------+

$regfile = "attiny2313.dat"                                 'ATtiny2313
$crystal = 16000000                                         '16MHz Quarz
$baud = 250000                                              'Baudrate 250Kbaud


Config Timer1 = Pwm , Pwm = 10 , Prescale = 8 , Compare A Pwm = Clear Down , Compare B Pwm = Clear Down


'Die Timer starten
Enable Timer1


'Subs
Declare Sub Adresse_basteln()

'IOs
Config Portd.4 = Input                                      'Adresse Bit 0
Config Portd.3 = Input                                      'Adresse Bit 1
Config Portd.6 = Input                                      'Adresse Bit 2
Config Portb.0 = Input                                      'Adresse Bit 3
Config Portb.1 = Input                                      'Adresse Bit 4
Config Portb.4 = Input                                      'Adresse Bit 5
Config Portb.5 = Input                                      'Adresse Bit 6
Config Portb.6 = Input                                      'Adresse Bit 7
Config Portb.7 = Input                                      'Adresse Bit 8

Config Portb.3 = Output                                     'PWM Out

Config Portd.2 = Output                                     'Statusled


'PullUps setzten
Set Portd.4
Set Portd.3
Set Portd.6
Set Portb.0
Set Portb.1
Set Portb.4
Set Portb.5
Set Portb.6
Set Portb.7


'Aliases
Statusled Alias Portd.2

Adressbit0 Alias Pind.4
Adressbit1 Alias Pind.3
Adressbit2 Alias Pind.6
Adressbit3 Alias Pinb.0
Adressbit4 Alias Pinb.1
Adressbit5 Alias Pinb.4
Adressbit6 Alias Pinb.5
Adressbit7 Alias Pinb.6
Adressbit8 Alias Pinb.7

Pwm_out Alias Pwm1a


'Funktionsvariablen                                          'Variable zum zusammenbasteln der DMX-Adresse
Dim Adresse As Integer

'DMX Variablen
Dim Dmx_count As Word                                       'Pakete die auf Ausgnge zielen
Dim Dmx_count_temp As Byte                                  'Gesendete Pakete
Dim Dmx_byte As Byte                                        'DMX Wert
Dim Dmx_clock As Byte                                       'Spielerei
Dim Dmx_adr As Word                                         'DMX-Adresse
Dim Dmx_adr_temp As Byte                                    'DMX-Adresse (Temp)
Dim Dmx_adr_c As Word                                       'DMX-Adresse (Counter)
Dim Dmx_off As Byte                                         'DMX-Signal noch da?

'Startwerte setzen
Set Statusled
Pwm_out = 0


' Uart initialiesieren
On Urxc Rec_isr
Enable Urxc

' Anfangsvariablen Festlegen
Dmx_count = 0                                               'kein Paket empfangen
Dmx_clock = 16                                              '

Enable Interrupts

' ******************************************************************************
' Hauptschleife
' ******************************************************************************
 Do
    Waitms 10
    Decr Dmx_off
    If Dmx_off = 0 Then
       Reset Statusled
    End If
    Call Adresse_basteln
 Loop

' ******************************************************************************
' Funktion: DMX-Signal emfangen und auswerten
'           Dieser Interupt wird beim eintreffen von seriellen Daten ausgefhrt
Rec_isr:
   $asm                                                     'Programmteil fr das Auswerten der
      .def Uart_status = R16                                'emfangenen Daten
      .def Uart_byte = R17
      .def Uart_temp1 = R18

            IN Uart_status,USR
            IN Uart_byte,UDR

            SBRC Uart_status,FE                             'Auswertung ob Datenbyte OK
            RJMP  Asm_reset                                 'bei berlnge des Datenbyte = DMX Reset
            RJMP  Asm_Loop1                                 'Daten OK--->

      Asm_reset:
            LDI Uart_temp1,$00                              'Adesszhler zurcksetzen
            STS {Dmx_count_temp} , Uart_temp1               'Daten dem Basicvariablen bergeben
            STS {Dmx_byte} , Uart_byte
            RJMP Asm_exit
      Asm_loop1:
            STS {Dmx_byte} , Uart_byte
      Asm_exit:
   $end Asm

   If Dmx_count_temp <> 0 Then                              'emfangene Daten im "Dmx_buf" ablegen
      Incr Dmx_count                                        'und den jeweiligen Kanal zuordnen
      If Dmx_adr_c = Dmx_count Then
         If Dmx_adr_temp < 2 Then
            Select Case Dmx_adr_temp
               Case 1:
                 If Dmx_byte < 255 Then
                   Pwm_out = Dmx_byte * 4
                 Else
                   Pwm_out = 1023
                 End If
            End Select
            Incr Dmx_adr_temp
            Incr Dmx_adr_c
         End If
      End If
   Else                                                     'bei DMX Reset Variablen setzen
      Dmx_count = 0 : Dmx_count_temp = 1
      Dmx_adr_c = Dmx_adr + 1 : Dmx_adr_temp = 1
      Decr Dmx_clock
      'nur eine Spielerei Toggelt eine LED am PortD.6 beim eintreffen von korrekten DMX Daten
      If Dmx_clock = 0 Then : Toggle Statusled : Dmx_clock = 16 : End If
   End If
   Dmx_off = 20
Return

Sub Adresse_basteln
  'Adresse zusammenbasteln
  Adresse = 0
  If Adressbit0 = 0 Then Adresse = Adresse + 1
  If Adressbit1 = 0 Then Adresse = Adresse + 2
  If Adressbit2 = 0 Then Adresse = Adresse + 4
  If Adressbit3 = 0 Then Adresse = Adresse + 8
  If Adressbit4 = 0 Then Adresse = Adresse + 16
  If Adressbit5 = 0 Then Adresse = Adresse + 32
  If Adressbit6 = 0 Then Adresse = Adresse + 64
  If Adressbit7 = 0 Then Adresse = Adresse + 128
  If Adressbit8 = 0 Then Adresse = Adresse + 256

  'Adresse speichern
  Dmx_adr = Adresse
End Sub